@@ -425,15 +425,19 @@ static void free_linkages(Sentence sent)
 		exfree(linkage->chosen_disjuncts, linkage->num_words * sizeof(Disjunct *));
 		free(linkage->link_array);
 
-		// XXX why isn't this in a string set ??
-		for (j=0; j<linkage->lifo.nwords; j++)
+		/* Q: Why isn't this in a string set ?? A: Because there is no
+		 * string-set handy when we  compute this */
+		if (linkage->disjunct_list_str)
 		{
-			if (linkage->lifo.disjunct_list_str[j])
-				free(linkage->lifo.disjunct_list_str[j]);
+			for (j=0; j<linkage->num_words; j++)
+			{
+				if (linkage->disjunct_list_str[j])
+					free(linkage->disjunct_list_str[j]);
+			}
+			free(linkage->disjunct_list_str);
 		}
-		free(linkage->lifo.disjunct_list_str);
 #ifdef USE_CORPUS
-		lg_sense_delete(&linkage->lifo);
+		lg_sense_delete(linkage);
 #endif
 
 		linkage_free_pp_info(linkage);
@@ -544,8 +548,12 @@ void partial_init_linkage(Linkage lkg, unsigned int N_words)
 	lkg->chosen_disjuncts = (Disjunct **) exalloc(lkg->cdsz * sizeof(Disjunct *));
 	memset(lkg->chosen_disjuncts, 0, N_words * sizeof(Disjunct *));
 
+	lkg->disjunct_list_str = NULL;
+#ifdef USE_CORPUS
+	lkg->sense_list = NULL;
+#endif
+
 	lkg->pp_info = NULL;
-	lkg->pp_violation = NULL;
 }
 
 void check_link_size(Linkage lkg)
@@ -583,25 +591,30 @@ static void post_process_linkages(Sentence sent, Parse_Options opts)
 	size_t N_linkages_post_processed = 0;
 	size_t N_valid_linkages = sent->num_valid_linkages;
 	size_t N_linkages_alloced = sent->num_linkages_alloced;
+	bool twopass = sent->length >= opts->twopass_length;
 
 	/* (optional) First pass: just visit the linkages */
-	/* The purpose of these two passes is to make the post-processing
-	 * more efficient.  Because (hopefully) by the time you do the
-	 * real work in the 2nd pass you've pruned the relevant rule set
+	/* The purpose of the first pass is to make the post-processing
+	 * more efficient.  Because (hopefully) by the time the real work
+	 * is done in the 2nd pass, the relevant rule set has been pruned
 	 * in the first pass.
 	 */
-	if (sent->length >= opts->twopass_length)
+	if (twopass)
 	{
 		for (in=0; in < N_linkages_alloced; in++)
 		{
 			Linkage lkg = &sent->lnkages[in];
 			Linkage_info *lifo = &lkg->lifo;
-			if (lifo->discarded || lifo->N_violations) continue;
+			if (lifo->discarded) continue;
 
+			/* We still need link names, even if there has been a morfo
+			 * violation. */
 			compute_link_names(lkg, sent->string_set);
-			post_process_scan_linkage(sent->postprocessor, opts, lkg);
+			if (lifo->N_violations) continue;
 
-			if ((9 == in%10) && resources_exhausted(opts->resources)) break;
+			post_process_scan_linkage(sent->postprocessor, lkg);
+
+			if ((49 == in%50) && resources_exhausted(opts->resources)) break;
 		}
 	}
 
@@ -614,6 +627,9 @@ static void post_process_linkages(Sentence sent, Parse_Options opts)
 
 		if (lifo->discarded) continue;
 
+		/* We need link names, even if morfo check fails */
+		if (!twopass) compute_link_names(lkg, sent->string_set);
+
 		/* N_violations could be non-zero if the linkage failed the
 		 * morphology check */
 		if (lifo->N_violations)
@@ -623,8 +639,7 @@ static void post_process_linkages(Sentence sent, Parse_Options opts)
 			N_linkages_post_processed++;
 			continue;
 		}
-		compute_link_names(lkg, sent->string_set);
-		ppn = do_post_process(sent->postprocessor, opts, lkg);
+		ppn = do_post_process(sent->postprocessor, lkg, twopass);
 	   post_process_free_data(&sent->postprocessor->pp_data);
 
 		if (NULL != ppn->violation)
@@ -642,6 +657,25 @@ static void post_process_linkages(Sentence sent, Parse_Options opts)
 		if ((9 == in%10) && resources_exhausted(opts->resources)) break;
 	}
 
+	/* If the timer expired, then we never finished post-processing.
+	 * Mark the remaining sentences as bad, as otherwise strange
+	 * results get reported.  At any rate, need to compute the link
+	 * names, as otherwise linkage_create() will crash and burn
+	 * trying to touch them. */
+	for (; in < N_linkages_alloced; in++)
+	{
+		Linkage lkg = &sent->lnkages[in];
+		Linkage_info *lifo = &lkg->lifo;
+		if (lifo->discarded) continue;
+		if (!twopass) compute_link_names(lkg, sent->string_set);
+		N_valid_linkages--;
+		lifo->N_violations++;
+
+		/* Set the message, only if not set (e.g. by sane_morphism) */
+		if (NULL == lifo->pp_violation_msg)
+			lifo->pp_violation_msg = "Timeout during postprocessing";
+	}
+
 	print_time(opts, "Postprocessed all linkages");
 
 	if (opts->verbosity > 1)
@@ -652,7 +686,6 @@ static void post_process_linkages(Sentence sent, Parse_Options opts)
 				N_valid_linkages, N_linkages_post_processed);
 	}
 
-	sent->num_linkages_alloced = N_linkages_alloced;
 	sent->num_linkages_post_processed = N_linkages_post_processed;
 	sent->num_valid_linkages = N_valid_linkages;
 }
