@@ -19,6 +19,7 @@
 #include "dict-common.h"
 #include "externs.h"
 #include "idiom.h"
+#include "pp_knowledge.h"
 #include "read-dict.h"
 #include "read-regex.h"
 #include "regex-morph.h"
@@ -57,7 +58,7 @@ static inline char * deinflect(const char * str)
  * has a pointer to an array of strings which are the punctuation/affix
  * names. */
 
-const char const * afdict_classname[] = { AFDICT_CLASSNAMES };
+const char *afdict_classname[] = { AFDICT_CLASSNAMES };
 #define NUMELEMS(a) (sizeof(a) / sizeof(a[0]))
 
 /**
@@ -84,28 +85,25 @@ Afdict_class * afdict_find(Dictionary afdict, const char * con, bool notify_err)
 
 #define AFFIX_COUNT_MEM_INCREMENT 64
 
-static void affix_list_resize(Afdict_class * ac)
-{
-	size_t old_mem_elems = ac->mem_elems;
-
-	ac->mem_elems += AFFIX_COUNT_MEM_INCREMENT;
-	ac->string = xrealloc((void *)ac->string, old_mem_elems,
-	      ac->mem_elems * sizeof(*ac->string));
-}
-
 static void affix_list_add(Dictionary afdict, Afdict_class * ac,
 		const char * affix)
 {
 	if (NULL == ac)  return; /* ignore unknown class name */
-	if (ac->length == ac->mem_elems)
-		affix_list_resize(ac);
+	if (ac->mem_elems <= ac->length)
+	{
+		size_t new_sz;
+		ac->mem_elems += AFFIX_COUNT_MEM_INCREMENT;
+		new_sz = ac->mem_elems * sizeof(const char *);
+		ac->string = (char const **) realloc((void *)ac->string, new_sz);
+	}
 	ac->string[ac->length] = string_set_add(affix, afdict->string_set);
 	ac->length++;
 }
 
 static void load_affix(Dictionary afdict, Dict_node *dn, int l)
 {
-	for (; NULL != dn; dn = dn->left)
+	Dict_node * dnx = NULL;	
+	for (; NULL != dn; dn = dnx)
 	{
 		char *string;
 		const char *con = word_only_connector(dn);
@@ -141,6 +139,9 @@ static void load_affix(Dictionary afdict, Dict_node *dn, int l)
 		affix_list_add(afdict, afdict_find(afdict, con,
 		               /*notify_err*/true), string);
 		free(string);
+
+		dnx = dn->left;
+		xfree((char *)dn, sizeof(Dict_node));
 	}
 }
 
@@ -185,17 +186,17 @@ static void get_dict_affixes(Dictionary dict, Dict_node * dn,
 		strncpy(w_last, w, w_len);
 		w_last[w_len] = '\0';
 
-		if ((INFIX_MARK == w_last[0]) &&
+		if ((infix_mark == w_last[0]) &&
 			 (0 != strcmp(w_last, EMPTY_WORD_MARK)))
 		{
 			affix_list_add(afdict, &afdict->afdict_class[AFDICT_SUF], w_last+1);
 		}
 		else
-		if (INFIX_MARK == w_last[w_len-1])
+		if (infix_mark == w_last[w_len-1])
 		{
 			w_last[w_len-1] = '\0';
 			affix_list_add(afdict, &afdict->afdict_class[AFDICT_PRE], w_last);
-			w_last[w_len-1] = INFIX_MARK;
+			w_last[w_len-1] = infix_mark;
 		}
 	}
 
@@ -217,7 +218,7 @@ static bool afdict_to_wide(Dictionary afdict, int classno)
 	wchar_t * wqs;
 	mbstate_t mbs;
 	size_t i;
-	int w;
+	size_t w;
 	dyn_str * qs;
 	const char *pqs;
 
@@ -235,17 +236,19 @@ static bool afdict_to_wide(Dictionary afdict, int classno)
 	pqs = qs->str;
 	memset(&mbs, 0, sizeof(mbs));
 	w = mbsrtowcs(NULL, &pqs, 0, &mbs);
-	if (0 > w)
+	if (0 > (int) w) /* mbsrtowcs returns ((size_t) -1) on error */
 	{
 		prt_error("Error: Affix dictionary: %s: "
 		          "Invalid utf8 character\n", afdict_classname[classno]);
 		return false;
 	}
 
-	/* Store the wide char version at the AFCLASS entry. */ 
-	ac->string = xrealloc((void *)ac->string, ac->mem_elems,
-	             sizeof(*wqs) * (w+1));
-	ac->mem_elems =  sizeof(*wqs) * (w+1);
+	/* Store the wide char version at the AFCLASS entry. */
+	if (ac->mem_elems <= w)
+	{
+		ac->mem_elems = w + 1;
+		ac->string = realloc(ac->string, sizeof(*wqs) * ac->mem_elems);
+	}
 	wqs = (wchar_t *)ac->string;
 	pqs = qs->str;
 	(void)mbsrtowcs(wqs, &pqs, w, &mbs);
@@ -266,18 +269,20 @@ static int cmplen(const void *a, const void *b)
 
 /**
  * Initialize several classes.
- * In case of a future dynamic change of the affix table, this
- * function needs to be invoked again after the affix table structure
- * is re-constructed (changes may be needed).
+ * In case of a future dynamic change of the affix table, this function needs to
+ * be invoked again after the affix table is re-constructed (changes may be
+ * needed - especially to first free memory and initialize the affix dict
+ * structure.).
  */
 static bool afdict_init(Dictionary dict)
 {
 	Afdict_class * ac;
 	Dictionary afdict = dict->affix_table;
 
-	/* FIXME: read_entry() builds word lists in reverse order (can we just create
-	 * the list top-down without breaking anything?). Unless it is fixed to
-	 * preserve the order, reverse here the word list for each affix class. */
+	/* FIXME: read_entry() builds word lists in reverse order (can we
+	 * just create the list top-down without breaking anything?). Unless
+	 * it is fixed to preserve the order, reverse here the word list for
+	 * each affix class. */
 	for (ac = afdict->afdict_class;
 		  ac < &afdict->afdict_class[NUMELEMS(afdict_classname)]; ac++)
 	{
@@ -295,12 +300,12 @@ static bool afdict_init(Dictionary dict)
 
 	/* Create the affix lists */
 	ac = AFCLASS(afdict, AFDICT_INFIXMARK);
-	if ((1 < ac->length) || ((1 == ac->length) && (1 != strlen(ac->string[0])))) 
+	if ((1 < ac->length) || ((1 == ac->length) && (1 != strlen(ac->string[0]))))
 	{
 		prt_error("Error: afdict_init: Invalid value for class %s in file %s"
 					 " (should have been one ASCII punctuation - ignored)\n",
 					  afdict_classname[AFDICT_INFIXMARK], afdict->name);
-		xfree((void *)ac->string, ac->mem_elems);
+		free((void *)ac->string);
 		ac->length = 0;
 		ac->mem_elems = 0;
 		ac->string = NULL;
@@ -344,14 +349,16 @@ static bool afdict_init(Dictionary dict)
 		int rc;
 
 		Regex_node * sm_re = (Regex_node *) malloc(sizeof(Regex_node));
-		char rebuf[MAX_WORD + 16] = "^((";
+		dyn_str *rebuf = dyn_str_new();
 
 		/* The regex is converted to: ^((original-regex)b)+$ */
-		strcat(rebuf, ac->string[0]);
-		strcat(rebuf, ")b)+$");
+		dyn_strcat(rebuf, "^((");
+		dyn_strcat(rebuf, ac->string[0]);
+		dyn_strcat(rebuf, ")b)+$");
+		sm_re->pattern = strdup(rebuf->str);
+		dyn_str_delete(rebuf);
 
 		afdict->regex_root = sm_re;
-		sm_re->pattern = strdup(rebuf);
 		sm_re->name = strdup(afdict_classname[AFDICT_SANEMORPHISM]);
 		sm_re->re = NULL;
 		sm_re->next = NULL;
@@ -442,7 +449,7 @@ dictionary_six_str(const char * lang,
 	dict->root = NULL;
 	dict->regex_root = NULL;
 	dict->word_file_header = NULL;
-	dict->exp_list = NULL;
+	dict->exp_list.exp_list = NULL;
 	dict->affix_table = NULL;
 	dict->recursive_error = false;
 	dict->version = NULL;
@@ -495,6 +502,7 @@ dictionary_six_str(const char * lang,
 			dict->afdict_class[i].string = NULL;
 		}
 	}
+	dict->affix_table = NULL;
 
 	/* Read dictionary from the input string. */
 	dict->input = input;
@@ -508,8 +516,6 @@ dictionary_six_str(const char * lang,
 	dict->pin = NULL;
 	dict->input = NULL;
 
-	dict->affix_table = NULL;
-
 	if (NULL == affix_name)
 	{
 		/*
@@ -541,22 +547,12 @@ dictionary_six_str(const char * lang,
 
 	dict->empty_word_defined = boolean_dictionary_lookup(dict, EMPTY_WORD_MARK);
 
-	dict->postprocessor   = post_process_open(pp_name);
-	dict->constituent_pp  = post_process_open(cons_name);
+	dict->base_knowledge  = pp_knowledge_open(pp_name);
+	dict->hpsg_knowledge  = pp_knowledge_open(cons_name);
 
 	dict->unknown_word_defined = boolean_dictionary_lookup(dict, UNKNOWN_WORD);
 	dict->use_unknown_word = true;
 
-#ifdef USE_FAT_LINKAGES
-	dict_node = dictionary_lookup_list(dict, ANDABLE_CONNECTORS_WORD);
-	if (dict_node != NULL) {
-		dict->andable_connector_set = connector_set_create(dict_node->exp);
-	} else {
-		dict->andable_connector_set = NULL;
-	}
-	free_lookup(dict_node);
-#endif /* USE_FAT_LINKAGES */
-
 	dict_node = dictionary_lookup_list(dict, UNLIMITED_CONNECTORS_WORD);
 	if (dict_node != NULL) {
 		dict->unlimited_connector_set = connector_set_create(dict_node->exp);
@@ -569,6 +565,7 @@ dictionary_six_str(const char * lang,
 
 failure:
 	string_set_delete(dict->string_set);
+	if (dict->affix_table) xfree(dict->affix_table, sizeof(struct Dictionary_s));
 	xfree(dict, sizeof(struct Dictionary_s));
 	return NULL;
 }
