@@ -5,7 +5,7 @@
 #include <map>
 #include <set>
 
-#include <link-grammar/link-includes.h>
+#include "link-includes.h"
 
 extern "C" {
 #include "count.h"
@@ -18,9 +18,11 @@ extern "C" {
 
 struct PositionConnector
 {
-  PositionConnector(Connector* c, char d, int w, int p, double cst, double pcst,
-                    bool lr, bool ll, const std::vector<int>& er, const std::vector<int>& el)
-    : connector(c), dir(d), word(w), position(p), cost(cst), parent_cost(pcst),
+  PositionConnector(Exp* e, Connector* c, char d, int w, int p, 
+                    double cst, double pcst, bool lr, bool ll,
+                    const std::vector<int>& er, const std::vector<int>& el)
+    : exp(e), connector(c), dir(d), word(w), position(p),
+      cost(cst), parent_cost(pcst),
       leading_right(lr), leading_left(ll),
       eps_right(er), eps_left(el)
   {
@@ -38,6 +40,9 @@ struct PositionConnector
     */
   }
 
+  // Original expression that this connector came from
+  Exp* exp;
+
   // Connector itself
   Connector* connector;
   // Direction
@@ -62,9 +67,10 @@ struct PositionConnector
 };
 
 
-// TODO: Hash connectors for faster matching
+// XXX TODO: Hash connectors for faster matching
 
-class WordTag {
+class WordTag
+{
 private:
   std::vector<PositionConnector> _left_connectors;
   std::vector<PositionConnector> _right_connectors;
@@ -139,7 +145,8 @@ public:
                          bool& leading_right, bool& leading_left,
                          std::vector<int>& eps_right,
                          std::vector<int>& eps_left,
-                         char* var, bool root, double parent_cost);
+                         char* var, bool root, double parent_cost,
+                         Exp* parent);
 
   // Caches information about the found matches to the _matches vector, and also
   // updates the _matches vector of all connectors in the given tag.
