@@ -41,6 +41,22 @@
 
 #include "error.h"
 
+#ifdef HAVE_ALLOCA_H
+# include <alloca.h>
+#elif defined __GNUC__
+# define alloca __builtin_alloca
+#elif defined _AIX
+# define alloca __alloca
+#elif defined _MSC_VER
+# include <malloc.h>
+# define alloca _alloca
+#else
+# include <stddef.h>
+# ifdef  __cplusplus
+extern "C"
+# endif
+void *alloca (size_t);
+#endif
 
 #ifdef _WIN32
 #include <windows.h>
@@ -67,7 +83,7 @@
 
 #endif /* _MSC_VER */
 
-/* Appearently, MinGW is also missing a variety of standard fuctions.
+/* Apparently, MinGW is also missing a variety of standard functions.
  * Not surprising, since MinGW is intended for compiling Windows
  * programs on Windows.
  * MINGW is also known as MSYS */
@@ -93,7 +109,7 @@ char * strndup (const char *str, size_t size);
  * cygwin just doesn't work very well. So we use our own custom version,
  * instead.
  */
-size_t lg_mbrtowc(wchar_t *, const char *, size_t, mbstate_t *);
+link_public_api(size_t) lg_mbrtowc(wchar_t *, const char *, size_t, mbstate_t *);
 #ifdef mbrtowc
 #undef mbrtowc
 #endif
@@ -153,16 +169,16 @@ int strncasecmp(const char *s1, const char *s2, size_t n);
 #if defined(_MSC_VER) || defined(__MINGW32__)
 #define assert(ex, ...) {                                                   \
 	if (!(ex)) {                                                             \
-		prt_error("\nAssertion ("#ex") failed at "FILELINE": "  __VA_ARGS__); \
-		fflush(stdout);                                                       \
+		prt_error("\nAssertion (" #ex ") failed at " FILELINE ": " __VA_ARGS__);  \
+		fprintf(stderr, "\n");                                                \
 		*((volatile int*) 0x0) = 42;  /* leave stack trace in debugger */     \
 	}                                                                        \
 }
 #else
 #define assert(ex, ...) {                                                   \
 	if (!(ex)) {                                                             \
-		prt_error("\nAssertion ("#ex") failed at "FILELINE": "  __VA_ARGS__); \
-		fflush(stdout);                                                       \
+		prt_error("\nAssertion (" #ex ") failed at " FILELINE ": " __VA_ARGS__);  \
+		fprintf(stderr, "\n");                                                \
 		__builtin_trap();  /* leave stack trace in debugger */                \
 	}                                                                        \
 }
@@ -179,11 +195,13 @@ int strncasecmp(const char *s1, const char *s2, size_t n);
 #define MAX(X,Y)  ( ((X) > (Y)) ? (X) : (Y))
 #endif
 
-/* Optimizations that only gcc undersatands */
+/* Optimizations etc. that only gcc undersatands */
 #if __GNUC__ > 2
 #define GNUC_MALLOC __attribute__ ((malloc))
+#define GNUC_UNUSED __attribute__ ((unused))
 #else
 #define GNUC_MALLOC
+#define GNUC_UNUSED
 #endif
 
 /**
@@ -286,7 +304,7 @@ static inline const char * skip_utf8_upper(const char * s)
 }
 
 /**
- * Return true if the intial upper-case letters of the
+ * Return true if the initial upper-case letters of the
  * two input strings match. Comparison stops when
  * both srings descend to lowercase.
  */
@@ -342,8 +360,6 @@ void init_memusage(void);
 void * xalloc(size_t) GNUC_MALLOC;
 void * exalloc(size_t) GNUC_MALLOC;
 
-void * xrealloc(void *, size_t oldsize, size_t newsize) GNUC_MALLOC;
-
 /* Tracking the space usage can help with debugging */
 #if defined(_WIN32) || __APPLE__
   /* **MUST** define for win32, Mac OSX, because xfree is listed in
